/*
    Constrained Local Model (CLM) sample code. A (partial)
    implementation of D. Christinacce and T. Cootes algorithm.

    Jesus Nuevo-Chiquero, University of Alcal. 2010
    jesus.nuevochiquero@gmail.com

    This file is part of 'CLM_uah'.

    CLM_uah is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    CLM_uah is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CLM_uah.  If not, see <http://www.gnu.org/licenses/>.

*/
#ifndef SHAPE_HH_
#define SHAPE_HH_

#include <cxcore.h>
#include <cv.h>

using namespace cv;

class ShapeBase : public PCA
{
public:
    int load(const string file);

    Mat m_eig_sqrt;
    
};


class Shape 
{

public:
    Shape() : m_updated(false){}
    Shape(ShapeBase sb): m_updated(false) {m_base = sb;
	init();
    }

    int load(const string file) {int r = m_base.load(file); if(!r)init(); return r;}

    Mat get();
    void set(Mat, bool limit = true);

    void set_parameters(const Mat b_g, const Mat st);
    void get_parameters(Mat&  b_s, Mat& st) const;

    ShapeBase m_base;

protected:

    Mat m_bs, m_st;

    Mat m_pts;

    bool m_updated;

    void init(void);

};

Mat project_over_ST(const Mat& sh, const ShapeBase& pdm);
void remove_ST(Mat sh, const Mat st_p);

#endif
