/*
    Constrained Local Model (CLM) sample code. A (partial)
    implementation of D. Christinacce and T. Cootes algorithm.

    Jesus Nuevo-Chiquero, University of Alcal. 2010
    jesus.nuevochiquero@gmail.com

    This file is part of 'CLM_uah'.

    CLM_uah is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    CLM_uah is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CLM_uah.  If not, see <http://www.gnu.org/licenses/>.

*/
#ifndef FITTING_HH_
#define FITTING_HH_

#include "cxcore.h"

#include "Shape.hh"
using namespace cv;

/** 
 * Fit the model to a given image.
 * 
 * @param img Image to fit
 * @param tmplt An image with the templates concatenated (H x(NW))
 * @param pdm A shape
 * 
 * @return The distance between the fitting model and the image.
 */
double fit_CLM( Mat& img, Mat& tmplt, 
	        Shape& pdm);

/** 
 * Evaluate the Zero-Normalized Cross-Correlation between the two
 * matrices. For images of the same size, this is faster than the
 * OpenCV's function (which uses DCT).
 * 
 * @param img An image
 * @param tmp An image
 * 
 * @return The ZNCC between the images.
 */
double eval_zncc(const Mat& img, const Mat& tmp);
#endif
