/*
    Constrained Local Model (CLM) sample code. A (partial)
    implementation of D. Christinacce and T. Cootes algorithm.

    Jesus Nuevo-Chiquero, University of Alcal. 2010
    jesus.nuevochiquero@gmail.com

    This file is part of 'CLM_uah'.

    CLM_uah is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    CLM_uah is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CLM_uah.  If not, see <http://www.gnu.org/licenses/>.

*/
#include <iostream>

#include "AppearanceBase.hh"

using namespace std;

int AppearanceBase::load(const string file)
{


    FileStorage fs(file, FileStorage::READ);
    FileNode rn; //Apperance base root node
    char str[128];

    if(fs.isOpened()){
	try{
	    rn = fs["AppearanceBase"];
	    
	    rn["per_patch_base"]>> m_per_patch;
	    rn["patch_count"]>> m_patch_count;
	}
	catch(Exception e){
	    cerr << "Bad file or tag not found. " << e.err << endl;
	    return -1;
	}

	if(m_per_patch){
	    
	    for(int ai=0; ai<m_patch_count; ai++){

		sprintf(str,"patch_%03d",ai);
		FileNode n = rn[(const char*)str];
		if(!n.empty()){
		    PCA pca;
		    Size sz;

		    FileNode node = n["patch_size"];
		    node[0] >> sz.width;
		    node[1] >> sz.height;
		    n["normalized"]>> m_normalized;

		    node = n["PCA"];
		    try{
			read(node["mean"], pca.mean);
			read(node["eigenvectors"], pca.eigenvectors);
			read(node["eigenvalues"], pca.eigenvalues);
			
		    }
		    catch(Exception e){
			cerr << e.err <<endl;
			return -1;
		    }
		    m_patch_base.push_back(pca);		    
		    m_patch_sz.push_back(sz);
		    Mat m;
		    sqrt(pca.eigenvalues.t(), m);
		    m_patch_eig_sqrt.push_back(m);
		}		
	    }
	    cout << m_patch_count << " patches read"<<endl;

	}
	else{
	    
	    Size sz;
	    FileNode node = rn["patch_size"];
	    node[0] >> m_sz.width;
	    node[1] >> m_sz.height;

	    rn["normalized"]>> m_normalized;
	    node = rn["PCA"];
	    if(!node.empty()){
		try{
		    read(node["mean"], m_base.mean);
		    read(node["eigenvectors"], m_base.eigenvectors);
		    read(node["eigenvalues"], m_base.eigenvalues);
		    
		    //transpose, because eigvals are 1 row
		    sqrt(m_base.eigenvalues.t(), m_eig_sqrt);
		}
		catch(Exception e){
		    cerr << e.err <<endl;
		    return -1;
		}
		cout << m_base.eigenvalues.rows << " eigenvalues read"<<endl;
	    }
	}
    }
    else
	return -1;


    

    return 0;
}
