 /*
    CLM_uah - Constrained Local Model (CLM) sample code. A (partial)
    implementation of D. Christinacce's and T. Cootes' algorithm.

    Jesus Nuevo-Chiquero, University of Alcal. 2010
    jesus.nuevochiquero@gmail.com

    This file is part of 'CLM_uah'.

    CLM_uah is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    CLM_uah is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CLM_uah.  If not, see <http://www.gnu.org/licenses/>.

*/
#include <sys/wait.h>
#include <time.h>
#include <sys/time.h>

#include <iostream>
#include <vector>

#include "cxcore.h"
#include "highgui.h"

//#include "basic_utils.hh"
//#include "utils.hh"
#include "utils_track_loss.hh"

using namespace std;

/************ ****** ******/
/* Random string (for /tmp/??? ******/

std::string get_random_string(unsigned int len)
{
    
    if(len>128)
	len = 128;


    struct timeval tv;
    gettimeofday(&tv,NULL);
    srand(tv.tv_usec);
    
    const char my_letters[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
    std::string str; 

    for(uint ai=0; ai<len; ai++){
	int k = (int)(rand()%strlen(my_letters));
	str.append(&(my_letters[k]),1);	
    }

    return str;
}


/*****************************
 *
 * Initialization with STASM
 *
 ****************************/

const static char *folder_STASM[]={".",
				   "../stasm2.4-MOD/linux", 
				   "../stasm/",
				   "/usr/local/bin",
				   "/usr/bin",
#ifdef STASM_INSTALL_DATA_DIR
				   STASM_INSTALL_DATA_DIR"/bin",
#endif
				   NULL
};

//return the location of stasm1shot
const char* find_STASM(void)
{
    char str[1024];
    for(int i=0; folder_STASM[i]!=NULL; i++){
	snprintf(str, 1024, "%s/stasm1shot", folder_STASM[i]);
	if(!access(str,X_OK))
	    return folder_STASM[i];

    }

    return NULL;
}


CvMat * stasm_find_points(const IplImage & k)
{
    
    CvMat *stasm_pts = NULL;
    int pidc;

    char pngfile[64];
    std::string rstr=get_random_string(5);
    sprintf(pngfile, "/tmp/smat_stasm_init_%s.jpg", rstr.c_str());
    char stasm_out[64];
    sprintf(stasm_out, "/tmp/smat_stasm_init_%s.xml", rstr.c_str());
    

    cvSaveImage(pngfile, &k);
    
    pidc = fork();
    if(pidc==0){//child
	const char *folder = find_STASM();
	chdir(folder);
	int ret = execl("stasm1shot", 
			"stasm1shot",  "-o", stasm_out, pngfile, NULL);

	//if we are here, an error occurred
	std::cerr << "WARNING: stasm1shot not found"<<std::endl;
	exit(-1);
    }
    else{
	waitpid(pidc, NULL, 0);
	
	//read the matrix
	CvMat *stasm = (CvMat*) cvLoad(stasm_out);
	if(CV_IS_MAT(stasm)){
	    
	    if(cvCountNonZero(stasm)== stasm->rows*stasm->cols){
	       stasm_pts = stasm;		
	    }
	    else{
		std::cerr << "STASM could not find a face: exiting" << std::endl;
		cvReleaseMat(&stasm);

		//		return -1;
	    }
	}
	
	unlink(pngfile);
	unlink(stasm_out);
	
    }
    
    return stasm_pts;

}





/*******************
 *
 * Check Similarity transformation values
 *
 ******************/




//quick and dirty solution. Not very good.
bool check_ST_values_ok(Mat _st, unsigned int frame, double &mean)
{
    
    CvMat st__ = _st;
    const CvMat *st = &(st__);

    static double ravg = cvmGet(st, 0,0);
    bool ok = true;
    double ang, sc;

    sc = sqrt(cvmGet(st,0,0)*cvmGet(st,0,0) +  cvmGet(st,1,0)*cvmGet(st,1,0));
    ang = atan2(cvmGet(st,1,0), cvmGet(st,0,0));


    //check the rotation is in the +-65 range

    if(fabs(ang)> M_PI/3)
	ok = false;

    if(frame>0 && ok){

	//     double m = cvmGet(st,0,0);

	if(fabs(sc - ravg)< ravg*.4){
	    ravg = ravg*(frame)/(frame+1) + sc/(frame+1);
	}
	else
	    ok = false;
    }

    if(frame < 100) //trust the first 100 frames
	ok = true;

    mean = ravg;

    return ok;
}

bool check_points_in_VJ(CascadeClassifier& haar, const IplImage *img, 
			const CvMat *shape, Rect& r)
{

    //scale image down
    int out = 0;
    double sd = 1;
    IplImage *k = cvCreateImage(cvSize(img->width/sd, img->height/sd), IPL_DEPTH_8U, 1);
    IplImage *k8;
 
    if(img->depth==32){
	k8 = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U,1);
	cvConvertScale(img, k8,255);
	cvResize(k8,k);
	cvReleaseImage(&k8);
    }
    else
	cvResize(img, k);

    vector<Rect> vr;
    haar.detectMultiScale(k, vr, 1.1,3,0,Size(40,40));
    cvReleaseImage(&k);
    
    if(vr.size())
	r = vr.at(0);
    else
	r.width =0;

    if(r.width>0){
	
	const CvMat *m = shape;

	//check that the extension of the shape is at least 2/5th of the size of the VJ square.
	CvMat subm;
	CvRect sr;
	double min, max;
	cvGetRow(m, &subm, 0);
	cvMinMaxLoc(&subm, &min, &max);
	sr.x = min;
	sr.width = max - sr.x;
	cvGetRow(m, &subm, 1);
	cvMinMaxLoc(&subm, &min, &max);
	sr.y = min;
	sr.height = max - sr.y;

	if(sr.width <= r.width/2. || sr.height <= r.width/2.)
	    return false;

	int limit = 1e6;
	// 	if(shape->get_robust()!=NONE)
	// 	    limit = pt->cols/2;
	// 	else
	limit = (m->cols-3)/3;
	
	for(int ai=0; ai<m->cols-1; ai++){//skip the 19th point
	    if(ai==8 || ai==13)
		continue;
	    double x = cvmGet(m, 0, ai)/sd;
	    double y = cvmGet(m, 1, ai)/sd;
	    if( x<r.x || x>r.width+r.x || y<r.y || y>r.y+r.height){      
		out++;
		if(out >= limit){		    
		    r.x *=sd;
		    r.y *=sd;
		    r.width *=sd;
		    r.height *=sd;
		    return false;
		}
	    }
	}
    }

    r.x *= sd;
    r.y *= sd;
    r.width *= sd;
    r.height *= sd;
    
#ifdef DEBUG
    std::cout << "check_points_in_VJ: ok ("<<r.x<<","<<r.y<<","<<r.width<<","<<r.height<<")"<<std::endl;
#endif

    return true;
}


// //Scale and rotation check
	
// void check_track_loss(boost::shared_ptr<IplImage> src, unsigned int frame)
// {
    
//     bool st_ok=true, vj_ok=true;
    
//     double mean;
//     CvMat *st = NULL;
//     CvRect vj_rect = cvRect(-1,-1,-1,-1);
//     if(m_shape->is_PDModel_valid()){
	    
// 	st = cvCreateMat(4,1,CV_32F);
// 	cvZero(st);
// 	boost::dynamic_pointer_cast<Shape>(m_shape)->get_coefficients(NULL, st);
// 	st_ok = check_ST_values_ok(st, frame, mean);
// 	cvReleaseMat(&st);
//     }

	
//     if(smat->m_shape->get_robust()!=NONE){
// 	if(!(frame%300))
// 	    vj_ok = check_points_in_VJ(m_haar.at(0), src.get(), m_shape, vj_rect);	    
// 	else if(cvCountNonZero(smat_mask.get())< 0.4*smat_mask->cols){
// 	    vj_ok = check_points_in_VJ(m_haar.at(0), src.get(), m_shape, vj_rect);	    
// 	    if(vj_rect.width == 0){//no face found
// 		//schecule periodical checks until the face is found
		    
// 	    }
// 	}
//     }
//     else{
// 	if(!(frame%15))
// 	    vj_ok = check_points_in_VJ(haar_front, src.get(), m_shape, vj_rect);	    
//     }
	
	
//     if(!st_ok || !vj_ok){
		    	   
// 	if(vj_rect.x==-1){
// 	    IplImage *src8 = cvCreateImage(cvGetSize(src.get()), IPL_DEPTH_8U,1);
// 	    cvConvertScale(src.get(), src8,255);
		
// 	    //V&J front
// 	    vj_rect = m_haar.at(0)->detect(src8, cvSize(floor(mean*0.6),floor(mean*0.6)), true);
// 	    cvReleaseImage(&src8);

// 	}
// 	if(vj_rect.width > 0){
// 	    if(CV_IS_MAT(st)){
// 		//reposition with (tx,ty) to the centre of VJ, rotation 0 and
// 		//Scale the mean of the values.
			    
// 		cvmSet(st, 0,0, mean);
// 		cvmSet(st, 1,0, 0);
// 		cvmSet(st, 2,0, vj_rect.x+(vj_rect.width)/2);
// 		cvmSet(st, 3,0, vj_rect.y+(2.*vj_rect.height/5.));
			    
// 		boost::dynamic_pointer_cast<Shape>(m_shape)->set_coefficients(NULL, st);
		
// 		resets_auto_vj.push_back(frame);
			    
// 		reset_rect_vj = vj_rect;
// 	    }
			
// 	    else{//use STASM
// 		//use STASM to reinitialize 		
// 		//write the frame to a PNG
// 		char pngfile[64];
// 		std::string rstr=get_random_string(5);
// 		sprintf(pngfile, "/tmp/smat_stasm_init_%s.jpg", rstr.c_str());

// 		cvSaveImage(pngfile,k);

// 		char stasm_out[64];
// 		sprintf(stasm_out, "/tmp/smat_stasm_init_%s.xml", rstr.c_str());

// 		int pidc;
		
// 		pidc = fork();
// 		if(pidc==0){//child

// 		    chdir("../../stasm2.4-MOD/linux");
// 		    execl("stasm1shot", 
// 			  "stasm1shot",  "-o", stasm_out, pngfile, NULL);
// 		}
// 		else{
// 		    waitpid(pidc, NULL, 0);

// 		    //read the matrix
// 		    CvMat *stasm = (CvMat*) cvLoad(stasm_out);
// 		    if(CV_IS_MAT(stasm)){
				    
// 			if(cvCountNonZero(stasm)== stasm->rows*stasm->cols){
					
// 			    cvScale(stasm, stasm,scale);//scale the points.
					
// 			    m_shape->set_shape(stasm);
// 			    cvReleaseMat(&stasm);
					
// 			    resets_auto_vj.push_back(frame);
					
// 			    reset_rect_vj = vj_rect;
// 			}
// 		    }
				
// 		    unlink(pngfile);
// 		    unlink(stasm_out);
				
// 		}			   
// 	    }
		
			
// 	    // #ifdef DEBUG
// 	    // 			if(wait_time)
// 	    // 			    wait_time = MAX(100, wait_time);
// 	    // #endif
						
// 	}
		    
//     }
		
//     if(CV_IS_MAT(st)){
// #ifdef DEBUG
// 	std::cout << "Scale= "<<cvmGet(st,0,0)<<", angle= " << cvmGet(st,1,0)<<std::endl;
// #endif
// 	cvReleaseMat(&st);
//     }
// }

